%%% Esempio di progetto di rete correttrice R(s) per G(s)

% G(s) = 0.2*(1-2*s)/(s*(1+10*s)*(1+0.1*s))

% Definisco la variabile complessa s di Laplace

s = tf('s');

% Introduco la funzione di trasferimento a tempo continuo

Gs=0.2*(1-2*s)/(s*(1+10*s)*(1+0.1*s));

% Esporto vettore dei polinomi la numeratore e denominatore 
% della funzione di trasferimento G(s)

[numGs,denGs]=tfdata(Gs,'v')

% Servono perché utilizzati in Simulink

 lsiminfo(ync,tsim)

% Una volta eseguita la simulazione in Simulink, vengono
% generati i vettori ync e tsim da cui posso valutare
% la massima sovraelongazione e il tempo di assestamento
% con la funzione Matlab lsiminfo

%%% Ora studio la stabilità del sistema in retroazione

Ga=Gs;

% Ho definito la funzione di anello Ga, che in questo caso coincide
% con G(s)

Gr=Ga/(1+Ga)

% Esporto i vettori in Matlab per verificare i poli della
% funzione di trasferimento in retroazione

[numGr,denGr]=tfdata(Gr,'v')

% Le radici per polinomio al denominatore sono i poli della
% funzione di trasferimento del sistema in retroazione Gr

roots(denGr)

% Si può notare che il sistema risulta stabile perché la parte 
% reale dei poli è negativa (sistema a tempo continuo) al più
% uno nullo, che garantisce l'errore a regime nullo in risposta 
% al gradino di riferimento unitario (sistema di 'tipo 1', ovvero
% 1 polo nell'origine)

roots(denGs)

% Posso anche verificare che i poli del sistema ad anello aperto
% sono stabili, tutti a parte reale negativa, a parte quello in
% zero, che rimane anche nella funzione di trasferimento in
% retroazione e che garantisce l'errore a regime nullo in risposta
% al gradino di riferimento

roots(numGs)

% Posso osservare che la radice del polinomio al numeratore, 
% ovvero lo zero della funzione di trasferimento G(s), è
% positivo, e che quindi si tratta di sistema a fase non
% minima. La caratteristica macroscopica di tali sistemi è
% che la risposta al gradino inizialmente presenta una sotto
% elongazione (il sistema risponde all'inizio con una uscita
% che ha segno opposto rispetto al segnale di ingresso), per
% poi inseguire il riferimento, perché si ha errore a
% regime nullo (polo nell'origine)

% Definisco la funzione di trasferimento del regolatore
% rete correttrice R(s) = (1+10*s)/(1+0.1*s)

Rs=(1+10*s)/(1+0.1*s)

% Esporto i vettori per l'implementazione in Simulink

[numRs,denRs]=tfdata(Rs,'v')

 lsiminfo(yc,tsim)

% Vedo il miglioramento in termini di tempo di assestamento
% e massima sovra-elongazione

% Definisco la funzione di anello, che ora vale il prodotto
% tra il sistema da controllare e il regolatore

Ga=Rs*Gs

% Confronto il diagramma di Nyquist delle funzioni di anello 
% del sistema non compensato Ga, con quello non compensato, Gs

nyquist(Ga,'r--',Gs,'b-')

% Osservo che il grafico del sistema compensato risulta 'più
% lontano' dal punto critico -1, e quindi è 'più stabile'

Grc=Ga/(1+Ga)

% Definisco la funzione di trasferimento del sistema in
% retroazione dopo l'inserimento del regolatore Gs

[numGrc,denGrc]=tfdata(Grc,'v')

% Esporto i vettori del polinomio, in particolare quello del
% denominatore, per verificare la stabilità del sistema in
% retroazione compensato

roots(denGrc)

% Osservo che tutti i poli sono a parte reale negativa, a parte 
% quello in zero che garantisce l'errore a regime nullo in
% risposta al gradino unitario di riferimento


